from typing import Literal, NewType, Dict, TYPE_CHECKING

from .teamtv_object import TeamTVObject

if TYPE_CHECKING:
    from .person import Person
    from .team import Team

Position = NewType("Position", Literal["ATTACK", "DEFENCE"])


class LineUp(TeamTVObject):
    @property
    def line_up_id(self):
        return self._line_up_id

    @property
    def sporting_event(self):
        return self._sporting_event

    def sync_players(self, team: "Team", positions: Dict["Person", Position]):
        # Make sure we only sync when things changed
        person_id_positions = {
            person.person_id: position for person, position in positions.items()
        }
        current_person_id_positions = {
            player["personId"]: player["position"]
            for player in self._roles_per_team.get(team.team_id, [])
        }

        if person_id_positions != current_person_id_positions:
            self._requester.request(
                "POST",
                f"/lineUps/{self.line_up_id}/resetTeamLineUp",
                {"teamId": team.team_id},
            )
            for person_id, position in person_id_positions.items():
                self._requester.request(
                    "POST",
                    f"/lineUps/{self.line_up_id}/addPlayer",
                    {
                        "teamId": team.team_id,
                        "personId": person_id,
                        "position": position,
                    },
                )

    def _use_attributes(self, attributes: dict):
        self._sporting_event = attributes["sportingEvent"]
        self._line_up_id = attributes["lineUpId"]
        self._roles_per_team = attributes["rolesPerTeam"]
        if not isinstance(self._roles_per_team, dict):
            self._roles_per_team = {}

        super()._use_attributes(attributes)

    def __repr__(self):
        return f"<LineUp sportingEvent={self.sporting_event}>"



import os.path

import pytest

from pyteamtv.exceptions import InputError


class TestVideoUpload:
    def test_new_video(self, current_team, test_mp4, requests_mock):
        """
        Test upload of a new video
        """
        sporting_event_id = "sporting-event-id-123"
        video_id = "video-id-123"

        requests_mock.get(
            f"https://fake-url/sportingEvents/{sporting_event_id}",
            json={
                "type": "training",
                "name": "Test Training",
                "sportingEventId": sporting_event_id,
                "clocks": {},
                "scheduledAt": "2022-01-01T10:00:00.000Z",
            },
        )

        sporting_event = current_team.get_sporting_event(
            sporting_event_id=sporting_event_id
        )

        file_size = os.path.getsize(test_mp4)

        adapter = requests_mock.post(
            f"https://fake-url/sportingEvents/{sporting_event_id}/initVideoUpload",
            json=video_id,
        )
        requests_mock.get(
            f"https://fake-url/videos/{video_id}",
            json={
                "videoId": video_id,
                "state": "new",
                "parts": [
                    {
                        "fileSize": file_size,
                        "tusUploadUrl": "https://upload-url/random-url",
                    }
                ],
            },
        )
        requests_mock.head(
            "https://upload-url/random-url",
            headers={
                "Upload-Offset": "0",
                "Upload-Length": str(file_size),
                "Tus-Resumable": "1.0.0",
            },
        )
        patch_adapter = requests_mock.patch(
            "https://upload-url/random-url",
            headers={
                "upload-offset": str(file_size),
                "Upload-Length": str(file_size),
                "Tus-Resumable": "1.0.0",
            },
        )

        sporting_event.upload_video(test_mp4, description="Test Training")

        assert adapter.last_request.json() == {
            "description": "Test Training",
            "files": [{"name": "test.mp4", "size": 991017}],
            "skipTranscoding": False,
            "tags": {},
        }

        assert len(patch_adapter.last_request.body) == file_size

    def test_resume_video(self, current_team, test_mp4, requests_mock):
        """
        Test resume of upload of an existing video.

        This test includes:
        1. Test tags are set when resume_if_exists is set
        2. Remote and local files match size
        3. File upload is resumed at based on Upload-Offset header
        """
        sporting_event_id = "sporting-event-id-123"
        tags = {"output_key": "main"}
        video_id = "video-id-123"

        requests_mock.get(
            f"https://fake-url/sportingEvents/{sporting_event_id}",
            json={
                "type": "training",
                "name": "Test Training",
                "sportingEventId": sporting_event_id,
                "clocks": {},
                "videoIds": [video_id],
                "scheduledAt": "2022-01-01T10:00:00.000Z",
            },
        )

        sporting_event = current_team.get_sporting_event(
            sporting_event_id=sporting_event_id
        )

        file_size = os.path.getsize(test_mp4)
        upload_offset = file_size - 100  # Fake the upload offset

        requests_mock.get(
            f"https://fake-url/videos",
            json=[
                {
                    "videoId": video_id,
                    "state": "new",
                    "tags": tags,
                    "parts": [
                        {
                            "fileSize": file_size
                            - 1,  # Return an incorrect filesize. The resume code MUST check
                            # if the local and remote filesizes match.
                            "state": "new",
                            "tusUploadUrl": "https://upload-url/random-url",
                        }
                    ],
                }
            ],
        )
        requests_mock.head(
            "https://upload-url/random-url",
            headers={
                "Upload-Offset": str(upload_offset),
                "Upload-Length": str(file_size),
                "Tus-Resumable": "1.0.0",
            },
        )
        adapter = requests_mock.patch(
            "https://upload-url/random-url",
            headers={
                "Upload-Offset": str(file_size),
                "Upload-Length": str(file_size),
                "Tus-Resumable": "1.0.0",
            },
        )

        with pytest.raises(
            InputError,
            match="When `resume_if_exists` is specified you must pass `tags`",
        ):
            sporting_event.upload_video(
                test_mp4,
                description="Test Training",
                resume_if_exists=True,
            )

        requests_mock.get(
            f"https://fake-url/videos/{video_id}",
            json={
                "videoId": video_id,
                "state": "new",
                "tags": tags,
                "parts": [
                    {
                        "fileSize": file_size,
                        "state": "new",
                        "tusUploadUrl": "https://upload-url/random-url",
                    }
                ],
            },
        )

        with pytest.raises(
            InputError,
            match=r"File size of '.*test\.mp4' doesn't match existing video",
        ):
            sporting_event.upload_video(
                test_mp4,
                description="Test Training",
                resume_if_exists=True,
                tags=tags,
            )

        requests_mock.get(
            f"https://fake-url/videos",
            json=[
                {
                    "videoId": video_id,
                    "state": "new",
                    "tags": tags,
                    "parts": [
                        {
                            "fileSize": file_size,
                            "state": "new",
                            "tusUploadUrl": "https://upload-url/random-url",
                        }
                    ],
                }
            ],
        )
        sporting_event.upload_video(
            test_mp4,
            description="Test Training",
            resume_if_exists=True,
            tags=tags,
        )

        assert len(adapter.last_request.body) == 100

    def test_resume_completed_video(self, current_team, test_mp4, requests_mock):
        """
        Test calling upload to finished video.
        """
        sporting_event_id = "sporting-event-id-123"
        tags = {"output_key": "main"}
        video_id = "video-id-123"

        requests_mock.get(
            f"https://fake-url/sportingEvents/{sporting_event_id}",
            json={
                "type": "training",
                "name": "Test Training",
                "sportingEventId": sporting_event_id,
                "clocks": {},
                "videoIds": [video_id],
                "scheduledAt": "2022-01-01T10:00:00.000Z",
            },
        )

        sporting_event = current_team.get_sporting_event(
            sporting_event_id=sporting_event_id
        )

        file_size = os.path.getsize(test_mp4)

        requests_mock.get(
            f"https://fake-url/videos",
            json=[
                {
                    "videoId": video_id,
                    "state": "new",
                    "tags": tags,
                    "parts": [
                        {
                            "fileSize": file_size,
                            "state": "upload-success",
                            "tusUploadUrl": "https://upload-url/random-url",
                        }
                    ],
                }
            ],
        )

        video = sporting_event.upload_video(
            test_mp4,
            description="Test Training",
            resume_if_exists=True,
            tags=tags,
        )

        assert video.video_id == video_id



import os
from typing import Optional, MutableMapping

from pyteamtv import TeamTVUser, TeamTVApp
from pyteamtv.exceptions import TokenMissing, ConfigurationError
from pyteamtv.models.access_requester import AccessRequester
from pyteamtv.models.resource_group.team import TeamResourceGroup


class App:
    def __init__(self, api):
        self.api = api
        if isinstance(api, TeamTVUser):
            self.membership_list = api.get_membership_list()
            self.current_resource_group: Optional[TeamResourceGroup] = None
        else:
            self.membership_list = None
            self.current_resource_group: TeamResourceGroup = api.get_resource_group()

        self.observation_logs = dict()
        self.sporting_events = dict()

    def get_access_requester(self) -> AccessRequester:
        return self.api.get_access_requester()

    def should_refresh(self, token: Optional[str]):
        if not token:
            return False
        return token != self.api.jwt_token

    def reset(self):
        self.observation_logs = dict()
        self.sporting_events = dict()

    def set_team(self, name: str):
        if not self.membership_list:
            raise Exception("Cannot set_team in TeamTVApp")

        if not self.current_resource_group or self.current_resource_group.name != name:
            self.current_resource_group = self.membership_list.get_membership_by_name(
                name=name
            ).resource_group  # type: TeamResourceGroup
            self.reset()

    def get_sporting_events(self):
        if not self.sporting_events:
            self.sporting_events = {
                sporting_event.sporting_event_id: sporting_event
                for sporting_event in sorted(
                    self.current_resource_group.get_sporting_events(),
                    key=lambda sporting_event: sporting_event.scheduled_at,
                    reverse=True,
                )
            }
        return self.sporting_events.values()

    def get_sporting_event(self, sporting_event_id: str):
        if sporting_event_id not in self.sporting_events:
            self.sporting_events[
                sporting_event_id
            ] = self.current_resource_group.get_sporting_event(sporting_event_id)
        return self.sporting_events[sporting_event_id]

    def get_observation_log(self, sporting_event_id: str):
        if sporting_event_id not in self.observation_logs:
            sporting_event = self.get_sporting_event(sporting_event_id)
            self.observation_logs[
                sporting_event_id
            ] = sporting_event.get_observation_log()
        return self.observation_logs[sporting_event_id]

    def get_sports_type(self) -> Optional[str]:
        tenant_id = self.current_resource_group.tenant_id
        for sports_type in [
            "korfball",
            "soccer",
            "hockey",
            "tennis",
            "handball",
            "volleybal",
        ]:
            if sports_type in tenant_id:
                return sports_type
        return None


def _get_current_app(
    app_id: Optional[str],
    session: MutableMapping,
    token: Optional[str],
    use_cache: bool = False,
    raise_on_missing_token: bool = True,
) -> Optional[App]:
    current_app: Optional[App] = session.get("current_app")

    if not current_app or current_app.should_refresh(token):
        if "TEAMTV_TOKEN" in os.environ:
            api = TeamTVUser(jwt_token=os.environ["TEAMTV_TOKEN"], use_cache=use_cache)
        else:
            if not app_id:
                app_id = os.environ.get("TEAMTV_APP_ID")
                if not app_id:
                    raise ConfigurationError("app_id must be set")

            if token:
                api = TeamTVApp(jwt_token=token, app_id=app_id, use_cache=use_cache)
            elif raise_on_missing_token:
                raise TokenMissing("Token not set")
            else:
                return None

        session["current_app"] = App(api=api)

    return session["current_app"]



import datetime
import time


def create_timestamp() -> str:
    now = datetime.datetime.utcnow()
    return datetime.datetime.isoformat(now).replace("+00:00", "Z")


class TestEventStream:
    def test_get_all_events(self, requests_mock, current_team):
        sporting_event_id = "sporting-event-id-123"
        event_stream_id = "event-stream-id-123"

        requests_mock.get(
            f"https://fake-url/sportingEvents/{sporting_event_id}",
            json={
                "type": "training",
                "name": "Test Training",
                "sportingEventId": sporting_event_id,
                "clocks": {},
                "scheduledAt": "2022-01-01T10:00:00.000Z",
            },
        )

        sporting_event = current_team.get_sporting_event(
            sporting_event_id=sporting_event_id
        )

        requests_mock.get(
            f"https://fake-url/sportingEvents/{sporting_event_id}/eventStreams",
            json=[
                {
                    "eventStreamId": event_stream_id,
                    "endpointUrls": {
                        "sse": f"https://fake-eventstream/streams/{event_stream_id}/subscribe/sse",
                        "polling": f"https://fake-eventstream/streams/{event_stream_id}/list",
                    },
                    "streamType": "all",
                }
            ],
        )

        streams = sporting_event.get_event_streams()

        requests_mock.get(
            f"https://fake-eventstream/streams/{event_stream_id}/list",
            json=[
                {
                    "event_attributes": {
                        "name": "Bla 1 - Hoi 1",
                        "awayTeam": {
                            "teamId": "d78b85ea-f0c4-11ed-a89a-254ed3a3b07d",
                            "appearance": {
                                "secondaryColor": None,
                                "primaryColor": None,
                                "logoUrl": None,
                            },
                            "name": "Bla 1",
                            "shortCode": "BLA",
                        },
                        "homeTeam": {
                            "teamId": "bd30a351-8527-4dcb-a6ff-57b92138fe03",
                            "appearance": {
                                "secondaryColor": None,
                                "primaryColor": None,
                                "logoUrl": None,
                            },
                            "name": "Hoi 1",
                            "shortCode": "HOI",
                        },
                        "scheduledAt": "2023-08-29T18:15:26.527000Z",
                    },
                    "event_name": "SportingEventCreated",
                    "event_id": "642270db37fdd48cb30572e26b2cfc50",
                    "occurred_on": create_timestamp(),
                },
            ],
        )

        time.sleep(0.001)
        with streams[0].open(seek_to_start=True) as stream:
            time.sleep(
                0.001
            )  # Make sure event timestamp below is after start of stream.open

            requests_mock.get(
                f"https://fake-eventstream/streams/{event_stream_id}/list?last-event-id=642270db37fdd48cb30572e26b2cfc50",
                json=[
                    {
                        "event_attributes": {
                            "period": "1",
                            "time": "1693333394.77399993",
                        },
                        "event_name": "StartPeriod",
                        "event_id": "ddfa76d51a72889c0d9f94f3da269199",
                        "occurred_on": create_timestamp(),
                    },
                ],
            )

            config, state, event = next(stream)
            assert event.event_type == "SportingEventCreated"
            assert not state.period_active

            config, state, event = next(stream)
            assert event.event_type == "StartPeriod"
            assert state.period_active

    def test_get_from_start(self, requests_mock, current_team):
        sporting_event_id = "sporting-event-id-123"
        event_stream_id = "event-stream-id-123"

        requests_mock.get(
            f"https://fake-url/sportingEvents/{sporting_event_id}",
            json={
                "type": "training",
                "name": "Test Training",
                "sportingEventId": sporting_event_id,
                "clocks": {},
                "scheduledAt": "2022-01-01T10:00:00.000Z",
            },
        )

        sporting_event = current_team.get_sporting_event(
            sporting_event_id=sporting_event_id
        )

        requests_mock.get(
            f"https://fake-url/sportingEvents/{sporting_event_id}/eventStreams",
            json=[
                {
                    "eventStreamId": event_stream_id,
                    "endpointUrls": {
                        "sse": f"https://fake-eventstream/streams/{event_stream_id}/subscribe/sse",
                        "polling": f"https://fake-eventstream/streams/{event_stream_id}/list",
                    },
                    "streamType": "all",
                }
            ],
        )

        streams = sporting_event.get_event_streams()

        requests_mock.get(
            f"https://fake-eventstream/streams/{event_stream_id}/list",
            json=[
                {
                    "event_attributes": {
                        "name": "Bla 1 - Hoi 1",
                        "awayTeam": {
                            "teamId": "d78b85ea-f0c4-11ed-a89a-254ed3a3b07d",
                            "appearance": {
                                "secondaryColor": None,
                                "primaryColor": None,
                                "logoUrl": None,
                            },
                            "name": "Bla 1",
                            "shortCode": "BLA",
                        },
                        "homeTeam": {
                            "teamId": "bd30a351-8527-4dcb-a6ff-57b92138fe03",
                            "appearance": {
                                "secondaryColor": None,
                                "primaryColor": None,
                                "logoUrl": None,
                            },
                            "name": "Hoi 1",
                            "shortCode": "HOI",
                        },
                        "scheduledAt": "2023-08-29T18:15:26.527000Z",
                    },
                    "event_name": "SportingEventCreated",
                    "event_id": "642270db37fdd48cb30572e26b2cfc50",
                    "occurred_on": create_timestamp(),
                },
            ],
        )

        time.sleep(0.001)

        with streams[0].open(seek_to_start=False) as stream:
            time.sleep(
                0.001
            )  # Make sure event timestamp below is after start of stream.open
            requests_mock.get(
                f"https://fake-eventstream/streams/{event_stream_id}/list?last-event-id=642270db37fdd48cb30572e26b2cfc50",
                json=[
                    {
                        "event_attributes": {
                            "period": "1",
                            "time": "1693333394.77399993",
                            "startPeriodAttributes": [],
                        },
                        "event_name": "StartPeriod",
                        "event_id": "ddfa76d51a72889c0d9f94f3da269199",
                        "occurred_on": create_timestamp(),
                    },
                    {
                        "event_attributes": {
                            "startPossessionAttributes": {"teamId": "team-id-123"}
                        },
                        "event_name": "StartPossession",
                        "event_id": "ddfa76d51a72889c0d9f94f3da269191",
                        "occurred_on": create_timestamp(),
                    },
                    {
                        "event_attributes": {
                            "startPossessionAttributes": {"teamId": "team-id-567"}
                        },
                        "event_name": "StartPossession",
                        "event_id": "ddfa76d51a72889c0d9f94f3da269191",
                        "occurred_on": create_timestamp(),
                    },
                    {
                        "event_attributes": {"periodCount": 3},
                        "event_name": "MatchConfigChanged",
                        "occurred_on": create_timestamp(),
                        "event_id": "7e59f7264fbc079fb4359a6f568f8bb7",
                    },
                ],
            )

            config, state, event = next(stream)
            assert event.event_type == "StartPeriod"
            assert state.period_active
            assert state.current_period == 1
            assert config.period_count == 2
            assert event.attributes["startPeriodAttributes"] == {}

            config, state, event = next(stream)
            assert state.current_possession_team_id == "team-id-123"

            config, state, event = next(stream)
            assert state.current_possession_team_id == "team-id-567"
            assert config.period_count == 2

            config, state, event = next(stream)
            assert event.event_type == "MatchConfigChanged"
            assert state.current_possession_team_id == "team-id-567"
            assert config.period_count == 3

    def test_get_last_event(self, requests_mock, current_team):
        sporting_event_id = "sporting-event-id-123"
        event_stream_id = "event-stream-id-123"

        requests_mock.get(
            f"https://fake-url/sportingEvents/{sporting_event_id}",
            json={
                "type": "training",
                "name": "Test Training",
                "sportingEventId": sporting_event_id,
                "clocks": {},
                "scheduledAt": "2022-01-01T10:00:00.000Z",
            },
        )

        sporting_event = current_team.get_sporting_event(
            sporting_event_id=sporting_event_id
        )

        requests_mock.get(
            f"https://fake-url/sportingEvents/{sporting_event_id}/eventStreams",
            json=[
                {
                    "eventStreamId": event_stream_id,
                    "endpointUrls": {
                        "sse": f"https://fake-eventstream/streams/{event_stream_id}/subscribe/sse",
                        "polling": f"https://fake-eventstream/streams/{event_stream_id}/list",
                    },
                    "streamType": "all",
                }
            ],
        )

        streams = sporting_event.get_event_streams()

        requests_mock.get(
            f"https://fake-eventstream/streams/{event_stream_id}/list",
            json=[
                {
                    "event_attributes": {
                        "name": "Bla 1 - Hoi 1",
                        "awayTeam": {
                            "teamId": "d78b85ea-f0c4-11ed-a89a-254ed3a3b07d",
                            "appearance": {
                                "secondaryColor": None,
                                "primaryColor": None,
                                "logoUrl": None,
                            },
                            "name": "Bla 1",
                            "shortCode": "BLA",
                        },
                        "homeTeam": {
                            "teamId": "bd30a351-8527-4dcb-a6ff-57b92138fe03",
                            "appearance": {
                                "secondaryColor": None,
                                "primaryColor": None,
                                "logoUrl": None,
                            },
                            "name": "Hoi 1",
                            "shortCode": "HOI",
                        },
                        "scheduledAt": "2023-08-29T18:15:26.527000Z",
                    },
                    "event_name": "SportingEventCreated",
                    "event_id": "642270db37fdd48cb30572e26b2cfc50",
                    "occurred_on": create_timestamp(),
                },
            ],
        )

        time.sleep(0.001)

        with streams[0].open(seek_to_start=False) as stream:
            time.sleep(
                0.001
            )  # Make sure event timestamp below is after start of stream.open
            requests_mock.get(
                f"https://fake-eventstream/streams/{event_stream_id}/list?last-event-id=642270db37fdd48cb30572e26b2cfc50",
                json=[
                    {
                        "event_attributes": {
                            "period": "1",
                            "time": "1693333394.77399993",
                            "startPeriodAttributes": [],
                        },
                        "event_name": "StartPeriod",
                        "event_id": "ddfa76d51a72889c0d9f94f3da269199",
                        "occurred_on": create_timestamp(),
                    },
                    {
                        "event_attributes": {
                            "startPossessionAttributes": {"teamId": "team-id-123"}
                        },
                        "event_name": "StartPossession",
                        "event_id": "ddfa76d51a72889c0d9f94f3da269191",
                        "occurred_on": create_timestamp(),
                    },
                    {
                        "event_attributes": {
                            "startPossessionAttributes": {"teamId": "team-id-567"}
                        },
                        "event_name": "StartPossession",
                        "event_id": "ddfa76d51a72889c0d9f94f3da269191",
                        "occurred_on": create_timestamp(),
                    },
                    {
                        "event_attributes": {"periodCount": 3},
                        "event_name": "MatchConfigChanged",
                        "occurred_on": create_timestamp(),
                        "event_id": "7e59f7264fbc079fb4359a6f568f8bb7",
                    },
                ],
            )

            # Make sure data is fetched
            time.sleep(1.5)

            config, state, event = stream.get_last_event()
            assert event.event_type == "StartPeriod"
            assert state.period_active
            assert state.current_period == 1
            assert config.period_count == 2

            config, state, event = stream.get_last_event()
            assert state.current_possession_team_id == "team-id-123"

            config, state, event = stream.get_last_event()
            assert state.current_possession_team_id == "team-id-567"
            assert config.period_count == 2

            config, state, event = stream.get_last_event()
            assert event.event_type == "MatchConfigChanged"
            assert state.current_possession_team_id == "team-id-567"
            assert config.period_count == 3

            assert stream.get_last_event() is None



from .teamtv_object import TeamTVObject


class Club(TeamTVObject):
    @property
    def name(self):
        return self._name

    @property
    def team_id(self):
        return self._club_id

    def _use_attributes(self, attributes: dict):
        self._club_id = attributes["clubId"]
        self._name = attributes["name"]

        super()._use_attributes(attributes)



from typing import Optional

from pyteamtv.infra.requester import Requester
from pyteamtv.models.list import List
from pyteamtv.models.membership import Membership
from pyteamtv.models.membership_list import MembershipList
from pyteamtv.models.sharing_group import SharingGroup

from .endpoint import API_ENDPOINT
from .token import decode
from ..exceptions import TeamNotFound, InputError
from ..models.access_requester import AccessRequester
from ..models.resource_group.team import TeamResourceGroup


class TeamTVUser(object):
    def __init__(self, jwt_token, use_cache: bool = False):
        self.jwt_token = jwt_token

        token = decode(
            jwt_token,
            verify=False,
        )

        self._requester = Requester(
            f"{API_ENDPOINT}/api", jwt_token, use_cache=use_cache
        )
        self.__token = token

    def get_access_requester(self) -> AccessRequester:
        print(self.__token)

    def get_membership_list(self):
        """
        :return: MembershipList
        """
        return MembershipList(
            Membership, self._requester, "GET", "/users/me/memberships"
        )

    def get_public_sharing_groups(self, sport_type: Optional[str] = None):
        return List(
            SharingGroup,
            self._requester,
            "GET",
            f"/sharingGroups?sportType={sport_type}"
            if sport_type
            else "/sharingGroups",
        )

    def get_team(
        self, name: Optional[str] = None, resource_group_id: Optional[str] = None
    ) -> TeamResourceGroup:
        membership_list = self.get_membership_list()
        if resource_group_id:
            membership = membership_list.get_membership_by_resource_group_id(
                resource_group_id
            )

            if not membership:
                raise TeamNotFound(
                    f"No team with resource group id '{resource_group_id}' found."
                )
        elif name:
            membership = membership_list.get_membership_by_name(name=name)

            if not membership:
                raise TeamNotFound(f"No team named '{name}' found.")
        else:
            raise InputError(f"Either `name` or `resource_group_id` must be specified.")

        return membership.resource_group

    def get_memberships(
        self, tenant_id: Optional[str] = None, type_: Optional[str] = None
    ) -> List[Membership]:
        return self.get_membership_list().get_memberships(tenant_id, type_)



from datetime import datetime, timezone
from threading import Event

import requests
import time
from queue import Queue


def match_event_fetcher(
    event_stream_url: str, match_event_queue: Queue, should_stop: Event
) -> None:
    last_event_id = None

    while not should_stop.is_set():
        url = event_stream_url
        if last_event_id:
            url += "?last-event-id={}".format(last_event_id)

        response = requests.get(url, timeout=2)

        events = response.json()
        for event in events:
            occurred_on = datetime.fromisoformat(
                event["occurred_on"].replace("Z", "+00:00")
            ).replace(tzinfo=timezone.utc)
            match_event_queue.put(
                dict(
                    eventType=event["event_name"],
                    eventAttributes=event["event_attributes"],
                    occurredOn=occurred_on,
                )
            )
            last_event_id = event["event_id"]

        time.sleep(1)



from .teamtv_object import TeamTVObject


class Person(TeamTVObject):
    @property
    def person_id(self):
        return self._person_id

    @property
    def name(self):
        return f"{self.first_name} {self.last_name}"

    @property
    def gender(self):
        return self._gender

    @property
    def first_name(self):
        return self._first_name

    @property
    def last_name(self):
        return self._last_name

    @property
    def tags(self):
        return self._tags

    def __hash__(self):
        return hash(self.person_id)

    def __eq__(self, other):
        return isinstance(other, Person) and other.person_id == self.person_id

    def _use_attributes(self, attributes: dict):
        self._person_id = attributes["personId"]
        self._first_name = attributes["firstName"]
        self._last_name = attributes["lastName"]
        self._gender = attributes["gender"]
        self._tags = attributes.get("tags", {})

        super()._use_attributes(attributes)

    def __repr__(self):
        return f"<Person name='{self.name}' person_id={self.person_id}>"



from typing import Type, TYPE_CHECKING

if TYPE_CHECKING:
    from pyteamtv.models.sporting_event import SportingEvent

from pyteamtv.infra.requester import Requester
from pyteamtv.models.list import List
from pyteamtv.models.observation import Observation


class ObservationLog(List[Observation]):
    def __init__(
        self,
        content_class: Type[Observation],
        requester: Requester,
        method: str,
        url: str,
        clock_id: str,
        sporting_event: "SportingEvent",
    ):
        super().__init__(content_class, requester, method, url)
        self._clock_id = clock_id
        self._sporting_event = sporting_event

    @property
    def sporting_event(self):
        return self._sporting_event

    def get_mapping_stats(self):
        stats = dict(success=0, failed=0)
        for observation in self:
            if observation.clock_id == self._clock_id:
                stats["success"] += 1
            else:
                stats["failed"] += 1
        return stats



from pathlib import Path

import setuptools
import builtins
from distutils.core import setup


builtins.__PYTEAMTV_SETUP__ = True
import pyteamtv


def setup_package():
    base_dir = Path(__file__).parent
    with open(base_dir / "README.md", "r") as f:
        readme = f.read()

    setup(
        name="pyteamtv",
        version=pyteamtv.__version__,
        author="Koen Vossen",
        author_email="info@koenvossen.nl",
        url="https://github.com/teamtv/pyteamtv",
        packages=setuptools.find_packages(exclude=["tests"]),
        license="GPL-3",
        description="API for TeamTV Platform",
        long_description=readme,
        long_description_content_type="text/markdown",
        classifiers=[
            "Intended Audience :: Developers",
            "Intended Audience :: Science/Research",
            "Programming Language :: Python :: 3",
            "Programming Language :: Python :: 3.6",
            "Programming Language :: Python :: 3.7",
            "Programming Language :: Python :: 3.8",
            "Programming Language :: Python :: 3.9",
            "Programming Language :: Python :: 3.10",
            "License :: OSI Approved",
            "Topic :: Scientific/Engineering",
        ],
        install_requires=[
            "requests>=2.0.0",
            "PyJWT>=1.7.1",
            "pyjwt[crypto]>=1.7.1",
            "tuspy==1.0.0",
        ],
        extras_require={
            "test": ["pytest", "pandas>=1.0.0", "requests-mock==1.10.0"],
            "cache": ["requests-cache>=0.9.7"],
        },
    )


if __name__ == "__main__":
    setup_package()

    del builtins.__PYTEAMTV_SETUP__



from .teamtv_object import TeamTVObject


class Video(TeamTVObject):
    @property
    def video_id(self):
        return self._video_id

    @property
    def parts(self):
        return self._parts

    @property
    def media_url(self):
        return self._media_url

    @property
    def state(self):
        return self._state

    @property
    def tags(self):
        return self._tags

    @property
    def skip_transcoding(self):
        return self._skip_transcoding

    @property
    def is_upload_completed(self):
        return all(part["state"] != "new" for part in self._parts)

    def __repr__(self):
        return f"<Video video_id={self.video_id} state={self.state}>"

    def _use_attributes(self, attributes: dict):
        self._video_id = attributes["videoId"]
        self._parts = attributes["parts"]
        self._media_url = attributes.get("mediaUrl")
        self._state = attributes["state"]
        self._tags = attributes.get("tags", {})
        self._skip_transcoding = attributes.get("skipTranscoding", False)

        super()._use_attributes(attributes)



from datetime import timedelta, datetime
from typing import List

from .event_store import Event
from ...models.match.match_config import MatchConfig, Team


def calculate_match_config(events: List[Event], timestamp: datetime) -> MatchConfig:
    match_config = MatchConfig.empty()

    for event in events:
        event_type = event.event_type
        event_attributes = event.attributes
        occurred_on = event.occurred_on

        if event_type == "SportingEventCreated":
            match_config.home_team = Team(
                team_id=event_attributes["homeTeam"]["teamId"],
                name=event_attributes["homeTeam"]["name"],
                short_code=event_attributes["homeTeam"]["shortCode"],
                primary_color=event_attributes["homeTeam"]["appearance"][
                    "primaryColor"
                ],
                secondary_color=event_attributes["homeTeam"]["appearance"][
                    "secondaryColor"
                ],
            )
            match_config.away_team = Team(
                team_id=event_attributes["awayTeam"]["teamId"],
                name=event_attributes["awayTeam"]["name"],
                short_code=event_attributes["awayTeam"]["shortCode"],
                primary_color=event_attributes["awayTeam"]["appearance"][
                    "primaryColor"
                ],
                secondary_color=event_attributes["awayTeam"]["appearance"][
                    "secondaryColor"
                ],
            )
            match_config.period_count = event_attributes.get("matchConfig", {}).get(
                "periodCount", 2
            )
        elif event_type == "MatchConfigChanged":
            if "periodCount" in event_attributes:
                match_config.period_count = event_attributes["periodCount"]

    return match_config



from datetime import datetime
from .teamtv_object import TeamTVObject


class BulkObservation(TeamTVObject):
    @property
    def bulk_observation_id(self) -> str:
        return self._bulk_observation_id

    @property
    def description(self) -> str:
        return self._description

    @property
    def observation_count(self) -> int:
        return self._observation_count

    @property
    def created(self) -> datetime:
        return self._created

    def _use_attributes(self, attributes: dict):
        self._bulk_observation_id = attributes["bulkObservationId"]
        self._description = attributes["description"]
        self._observation_count = attributes["observationCount"]
        self._created = datetime.fromisoformat(
            attributes["created"].replace("Z", "+00:00")
        )

        super()._use_attributes(attributes)
